#! /bin/sh
# replacement to download ide from a zip

# configurable user
ALERT_RECIPIENT=""

# configure the ALERT_RECIPIENT to the address that receives the alerts,
# then remove these four lines, and install this script
echo UNCONFIGURED
exit 1


PATH=/usr/local/bin:/bin:/usr/bin:/usr/sbin:/usr/local/sbin
export PATH

SAV_CONF="/etc/sav.conf"
VERBOSE_FLAG=""
DATE_FMT="+%d %b %Y %H:%M:%S"

NEW_VERSION=""
LOCKED=""
NORETRY=""
if [ -t 0 ]; then
	NORETRY="not retrying when issued on a terminal"
fi
for arg do
	case "$arg" in
	[nN][eE][wW])
		NEW_VERSION=" (new)"
		;;
	locked)
		LOCKED=1
		echo OK # pending result for locking
		;;
	noretry)
		NORETRY="no retry"
		;;
	esac
done

# default timeout (possibly exported by run_get_ides)
#
if [ -z "$GET_IDES_TO" ]; then
	GET_IDES_TO=900
	export GET_IDES_TO
fi	

# get SAV config directory
#
if [ -z "$GET_IDES_SAV_DIR" ]; then
	printf 'get_ides: getting config        '
	GET_IDES_SAV_DIR=`egrep 'SAV (virus data )?[Dd]ir(ectory)? *=' $SAV_CONF |\
		sed 's/^[^=]*=[ ]*//'`
	echo "$GET_IDES_SAV_DIR"
	export GET_IDES_SAV_DIR
fi

DOWNLOAD_DIR="$GET_IDES_SAV_DIR/downloads"
if [ ! -d "$DOWNLOAD_DIR" ]; then
	mkdir "$DOWNLOAD_DIR"
	if [ -w / ]; then
		chown --reference="$GET_IDES_SAV_DIR" "$DOWNLOAD_DIR"
	fi
fi
SAVE_IDE="$GET_IDES_SAV_DIR/save_ide"
test -d "$SAVE_IDE" || mkdir "$SAVE_IDE"

cd $DOWNLOAD_DIR
if [ $? -ne 0 ]; then
	echo "cannot cd $DOWNLOAD_DIR.  aborted."
	echo "# subject: SYS ERROR" >&2
	exit 4
fi

# set current version number and lock it
#
if [ -z "$LOCKED" ]; then
	printf 'get_ides: acquiring lock        '
	touch sweep_version
	if [ -w / ]; then
		chown --reference=. sweep_version
	fi
	lockmail -t $GET_IDES_TO sweep_version $0 $@ locked
	rtc=$?
	if [ $rtc -eq 75 ]; then
		echo "EX_TEMPFAIL"
	fi
	exit $rtc
fi

# locked on sweep_version, continue.

# get zip version of ide files we should have
#
printf 'get_ides: current zip version   '
ZIP_VERSION=`cat sweep_version`
ZIP="${ZIP_VERSION}_ides.zip"
if [ -n "$NEW_VERSION" ]; then
	echo "" > sweep_version
	rm -f "$ZIP"
	ZIP_VERSION=""
fi
if [ -z "$ZIP_VERSION" ]; then
	ZIP_VERSION=`sweep -v | sed -n '/Product version/{
s|^[^:]*:[ 	]*||
s|\.||
s|[. 	].*$||
p
}'`
	echo "$ZIP_VERSION" > sweep_version
	if [ -w / ]; then
		chown --reference=. sweep_version
	fi
fi

ZIP="${ZIP_VERSION}_ides.zip"
ZIPURL="http://www.sophos.com/downloads/ide/$ZIP"
echo "${ZIP}${NEW_VERSION}"

# download zip file
#
CURL_FLAGS="$VERBOSE_FLAG --silent --show-error \
	--remote-name --remote-time \
	--max-time $GET_IDES_TO \
	--user-agent avfilter/curl \
	--write-out http_code=%{http_code};time_total=%{time_total}"
lsZIP=""
if [ -f "$ZIP" ]; then
	printf 'get_ides: current zip file date '
	date --reference="$ZIP" "$DATE_FMT"
	CURL_FLAGS="$CURL_FLAGS --time-cond $ZIP"
	lsZIP="$ZIP"
fi
printf 'get_ides: downloading zip file  '
vars=`curl $CURL_FLAGS $ZIPURL`
rtc=$?
if [ $rtc -eq 0 ]; then
	eval $vars
	echo "$http_code (in $time_total secs.)"
else
	echo ""
fi

if [ $rtc -ne 0 -o ! -f $ZIP ]; then
	echo "curl failed to get $ZIP: rc=$rtc.  aborted."
	echo "# subject: CURL ERROR" >&2
	echo ""
	echo "working directory:"
	ls -ld "`pwd`" "$lsZIP"
	echo ""
	id
	exit 5
fi

if [ "$http_code" -ge 400 ]; then
	echo "curl failed to get $ZIP: http=$http_code.  aborted."
	echo "# subject: HTTP ERROR" >&2
	exit 5
fi

# it may happen that the file hasn't been canged yet.
# we schedule a retry by sending a new mail msg like this,
# in the new message we set no further retries as well.
if [ "$http_code" -eq 304 ]; then
	if [ -z "$NORETRY" ]; then
		printf '\nScheduling a retry\n'
		at now + 5 minute 2>&1 <<-endofjob
			sendmail "$ALERT_RECIPIENT" <<-endofmail
			From: $0
			To: $ALERT_RECIPIENT
			Subject: Retried sophos download
			Date: \`date --rfc-2822\`
			X-retry-further: none
			MIME-Version: 1.0
			Content-Type: text/plain; charset=ISO-8859-1

			curl $CURL_FLAGS $ZIPURL
			wrote out: $vars
			
			Scheduled at `date` for execution at \`date\`.
			endofmail
		endofjob
		rtc=$?
		if [ $rtc -eq 0 ]; then
			echo "### subject: will retry" >&2
		else
			echo "subject: Retry failed" >&2
			echo "rtc = $rtc from at"
		fi
	else
		echo "### subject: 304: not modified" >&2
		echo "$NORETRY"
	fi
	exit 0;
fi
if [ "$http_code" -ne 200 ]; then
	echo "$http_code: unknown result"
	echo "# subject: HTTP ERROR" >&2
	exit 1;
fi

printf 'get_ides: testing zip file      '
unzip -q -t $ZIP > /dev/null
if [  $? -ne 0 ]; then
	echo "BAD"
	echo "# subject: BAD ZIP" >&2
	rm -f "bad-$ZIP"
	mv "$ZIP" "bad-$ZIP"
	exit 5
fi
echo "OK"

# show zip date
#
printf 'get_ides: new zip date          '
date --reference="$ZIP" "$DATE_FMT"

# remove ides of previous version
#
if [ -n "$NEW_VERSION" ]; then
	rm -f "../*.ide"
fi

printf 'get_ides: inflating zip file    '
unzip -q -d .. -o $ZIP
if [  $? -ne 0 ]; then
	echo "error"
	echo "# subject: unzip ERROR" >&2
else
	echo "done"
fi

if [ -w / ]; then
	printf 'get_ides: checking ownership    '
	chown --verbose --reference=. "$ZIP"
fi

# reload ides (on new_version, courierfilter has already restarted)
avfilter_sig reload virus
if [  $? -ne 0 ]; then
	echo "subject: AV ERROR" >&2
fi
