/*
** tfork.c - written in milano by vesely on 19 feb 2004
** test what does sophos anti virus engine after fork
*/
#if defined(HAVE_CONFIG_H)
#include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <assert.h>
#include <errno.h>
#include <ctype.h>
#include <limits.h>

#include <sys/types.h>
#include <sys/wait.h>
#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#include <signal.h>
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

/* SAV */
#include <sophos/compute.h>
#include <sophos/savitype.h>
/*
 * -ale: INITGUID is the declare/define interface switch
 */
#define INITGUID
#include <sophos/csavi3c.h>
#undef INITGUID


static CISavi3 *pSAVI;
static int live_children = 0;
static const char test_option[] = "Mime";
static const int test_option_type = 3;

static void child_reaper(int sig)
{
	int status;
	(void)sig;
	
	while (waitpid(-1, &status, WNOHANG) > 0)
	{
		--live_children;
	}
}

static void child_wait_init(void)
{
	struct sigaction act;
	memset(&act, 0, sizeof act);
	sigemptyset(&act.sa_mask);
	
	act.sa_flags = SA_NOCLDSTOP;
	act.sa_handler = child_reaper;
	sigaction(SIGCHLD, &act, NULL);
}

static int get_test_option(void)
{
	char buf[80];
	U32 size = sizeof buf / 2;
	HRESULT hr = pSAVI->pVtbl->
		GetConfigValue(pSAVI, test_option, test_option_type,
			sizeof buf, buf, &size);
	if (SOPHOS_FAILED(hr) || size > 2)
	{
		fprintf(stderr,
			"tfork: cannot get %s[%d]: size=%u [0x%08lx]\n",
			test_option, test_option_type, (unsigned int)size, (unsigned long)hr);
		return -1;
	}
	return buf[0] - '0';
}

static int set_test_option(int value)
{
	int rtc = 0;
	HRESULT hr;
	char buf[80];
	
	sprintf(buf, "%d", value);
	hr = pSAVI->pVtbl->
		SetConfigValue(pSAVI, test_option, test_option_type, buf);
	if (SOPHOS_FAILED(hr) || strlen(buf) != 1)
	{
		fprintf(stderr,
			"tfork: cannot set %s[%d]=%s: [0x%08lx]\n",
			test_option, test_option_type, buf, (unsigned long)hr);
		rtc = 1;
	}
	return rtc;
}

static void change_test_option(char const *whoami)
{
	int value = get_test_option();
	if (value == 0 || value == 1)
	{
		int new_value = 1 - value;
		fprintf(stdout,
			"%s: changing option %s from %d to %d\n",
				whoami, test_option, value, new_value);
		if (set_test_option(new_value) == 0)
		{
			value = get_test_option();
			if (value >= 0)
				fprintf(stdout,
					"%s: now option %s is %d\n",
						whoami, test_option, value);
		}
		fflush(stdout);
	}
	else
		fprintf(stderr, "tfork: unexpected value %d\n", value);
}

static void check_test_option(char const *whoami)
{
	int value = get_test_option();
	if (value == 0 || value == 1)
	{
		if (value >= 0)
			fprintf(stdout,
				"%s: option %s is %d\n",
					whoami, test_option, value);
		fflush(stdout);
	}
	else
		fprintf(stderr, "tfork: unexpected value %d\n", value);
}

static void do_test(void)
{
	int ctl[2];
	pid_t pid;
	char buf[4];

	if (pipe(ctl))
	{
		perror("pipe");
		return;
	}
	
	pid = fork();
	if (pid < 0)
	{
		perror("fork");
		return;
	}
	
	if (pid == 0) // child
	{
		int rtc;
		close(ctl[1]);
		rtc = read(ctl[0], buf, 1); // wait for parent
		if (rtc == 1)
			change_test_option("child");
		else
			perror("read ctl");
		exit(0);
	}
	else // parent
	{
		live_children = 1;
		close(ctl[0]);
		check_test_option("parent");
		if (write(ctl[1], buf, 1) != 1)
			perror("write ctl");
		while (live_children > 0)
			sleep(1);
		check_test_option("parent");
	}
}

/*
 * Function: InitialiseSAVI
 *
 * Parameters: (1) pointer to the SAVI client name.
 *
 * Returns: Initialised SAVI interface ready for use
 *          on success.
 *          NULL on failure.
 *
 * Purpose: Performs the bootstrap process -- creates
 *          the SAVI interface from scratch, and
 *          initialises it ready for use.
 */
static CISavi3 *InitialiseSAVI(LPCOLESTR ClientName)
{
  CISavi3              *pSAVI = NULL;
  CISweepClassFactory2 *pFactory =NULL;
  HRESULT               hr;
    
  /*
   * Request a class factory interface.
   */
  hr = DllGetClassObject((REFIID)&SOPHOS_CLASSID_SAVI,
                         (REFIID)&SOPHOS_IID_CLASSFACTORY2,
                         (void **)&pFactory);
  if (SOPHOS_FAILED(hr))                                                                          
  {
    fprintf(stderr, "ERROR: Failed to get class factory interface [0x%08lx].\n",
    	(unsigned long)hr);
    return NULL;
  }

  /*
   * Ask the class factory for a CSAVI3 interface.
   */
  hr = pFactory->pVtbl->CreateInstance(pFactory,
                                       NULL,
                                       &SOPHOS_IID_SAVI3,
                                       (void **)&pSAVI);

  /*
   * Drop the factory immediately, we don't need it
   * again in this example.
   */
  pFactory->pVtbl->Release(pFactory);

  /*
   * Did we get the interface we requested?
   */
  if (SOPHOS_FAILED(hr))
  {
    fprintf(stderr, "ERROR: Failed to get a CSAVI3 interface [0x%08lx].\n",
    	(unsigned long)hr);
    return NULL;
  }

  /*
   * Ask SAVI to initialise itself.
   */
	hr = pSAVI->pVtbl->InitialiseWithMoniker(pSAVI, ClientName);

  /*
   * If the initialisation failed, then release the SAVI
   * interface and set the pointer to NULL.
   */
  if (SOPHOS_FAILED(hr))
  {
    fprintf(stderr, "ERROR: Failed to initialise SAVI [0x%08lx].",
    	(unsigned long)hr);
    pSAVI->pVtbl->Release(pSAVI);
    pSAVI = NULL;
  }

  return pSAVI;
}

int main(int argc, char *argv[])
{
	int i;
	int load = 0;
	for (i = 1; i < argc; ++i)
		if (strcmp(argv[i], "-l") == 0)
			load = 1;
			
	child_wait_init();
	pSAVI = InitialiseSAVI("tfork");
	if (pSAVI)
	{
		if (load)
		{
			HRESULT hr = pSAVI->pVtbl->LoadVirusData(pSAVI);
			if (SOPHOS_FAILED(hr))
				fprintf(stderr,
					"tfork: LoadVirusData error: %#lx\n",
					(unsigned long) hr);
		}

		do_test();
		
		pSAVI->pVtbl->Terminate(pSAVI);
		pSAVI->pVtbl->Release(pSAVI);
	}
	return 0;
}
